package com.xc.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.xc.dao.*;
import com.xc.pojo.*;
import com.xc.pub.redismessage.service.MessageService;
import com.xc.service.*;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xc.common.ServerResponse;
import com.xc.service.*;
import com.xc.utils.DateTimeUtil;
import com.xc.utils.KeyUtils;
import com.xc.utils.email.SendHTMLMail;
import com.xc.utils.pay.GpayUtil;
import com.xc.utils.redis.RedisShardedPoolUtils;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;

import com.xc.vo.pay.GPayVO;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service("iUserRechargeService")
public class UserRechargeServiceImpl implements IUserRechargeService {
    private static final Logger log = LoggerFactory.getLogger(UserRechargeServiceImpl.class);

    @Autowired
    UserRechargeMapper userRechargeMapper;

    @Autowired
    IUserService iUserService;

    @Autowired
    UserMapper userMapper;

    @Autowired
    IAgentUserService iAgentUserService;
    @Autowired
    AgentUserMapper agentUserMapper;
    @Autowired
    ISiteSettingService iSiteSettingService;
    @Autowired
    UserCashDetailMapper userCashDetailMapper;
    @Autowired
    ISiteInfoService iSiteInfoService;
    @Autowired
    IStockCoinService iStockCoinService;

    @Autowired
    private MessageService messageService;

    @Autowired
    SitePayMapper sitePayMapper;

    @Autowired
    SiteTaskLogMapper siteTaskLogMapper;

    public ServerResponse checkInMoney(int maxOrder, Integer userId) {
        int count = this.userRechargeMapper.checkInMoney(0, userId);

        if (count > maxOrder) {
            return ServerResponse.createByErrorMsg("一小时内只能发起" + maxOrder + "次入金");
        }
        return ServerResponse.createBySuccess();
    }


    public ServerResponse inMoney( Integer id ,String amtRMB, String amt, String payType, HttpServletRequest request) {
        if (StringUtils.isBlank(amtRMB) || StringUtils.isBlank(amt) || StringUtils.isBlank(payType)) {
            return ServerResponse.createByErrorMsg("参数不能为空");
        }
        SiteSetting siteSetting = this.iSiteSettingService.getSiteSetting();
        if (siteSetting == null) {
            return ServerResponse.createByErrorMsg("设置set未初始化");
        }
        if ((new BigDecimal(siteSetting.getChargeMinAmt() + "")).compareTo(new BigDecimal(amtRMB)) == 1) {
            return ServerResponse.createByErrorMsg("充币金额不得低于" + siteSetting.getChargeMinAmt());
        }
        SiteInfo siteInfo = null;
        ServerResponse serverResponseInfo = this.iSiteInfoService.getInfo();
        if (serverResponseInfo.isSuccess()) {
            siteInfo = (SiteInfo) serverResponseInfo.getData();
            if (StringUtils.isBlank(siteInfo.getSiteHost()) ||
                    StringUtils.isBlank(siteInfo.getSiteEmailTo())) {
                return ServerResponse.createByErrorMsg("请先设置Host and ToEmail");
            }
        } else {
            return serverResponseInfo;
        }
        User user = this.iUserService.getCurrentRefreshUser(request);
        if (user.getIsActive() != 2) {
            return ServerResponse.createByErrorMsg("未实名认证不能发起充币");
        }
//        ServerResponse serverResponse = checkInMoney(10, user.getId());
//        if (!serverResponse.isSuccess()) {
//            return serverResponse;
//        }
        //查询支付通道
        SitePay sitePay = sitePayMapper.selectByPrimaryKey(id);
        UserRecharge userRecharge = new UserRecharge();
        userRecharge.setUserId(user.getId());
        userRecharge.setNickName(user.getRealName());
        userRecharge.setAgentId(user.getAgentId());
        userRecharge.setAgentName(user.getAgentName());
        userRecharge.setPhone(user.getPhone());
        String ordersn = KeyUtils.getRechargeOrderSn();
        userRecharge.setOrderSn(ordersn);
        userRecharge.setPaySn(id.toString());
        userRecharge.setPayChannel(payType);
        userRecharge.setPayAmtRMB(new BigDecimal(amtRMB));
        userRecharge.setPayAmt(new BigDecimal(amt));
        userRecharge.setOrderStatus(0);
        userRecharge.setAddTime(new Date());
        userRecharge.setOrderDesc(sitePay.getChannelType());
//        BigDecimal payAmt = new BigDecimal(amt);
        // 账户充币
        //汇率
//        CoinAdminListVO coinAdminListVO = this.iStockCoinService.getSelectCoin();
//        if (0 == coinAdminListVO.getDynamicRate()){
//            userRecharge.setPayAmt(userRecharge.getPayAmtRMB().divide(new BigDecimal(coinAdminListVO.getNowPrice()),2,BigDecimal.ROUND_DOWN));
//        }else {
//            userRecharge.setPayAmt(new BigDecimal(amt));
//        }
        int insertCount = this.userRechargeMapper.insert(userRecharge);
        JSONObject jb;
        if("公户".equals(sitePay.getChannelName())){
            jb = GpayUtil.gPay(amtRMB , ordersn , user.getRealName());
        }else if("365支付".equals(sitePay.getChannelName())){
            jb = GpayUtil.gPay365(amtRMB , ordersn , user.getRealName(),user.getId()+"");
        }else {
            jb = GpayUtil.gPay2(amtRMB , ordersn , user.getRealName());
        }
        String msg = jb.getString("msg");
        String payUrl = null;
        if("365支付".equals(sitePay.getChannelName())){
            if("200".equals(jb.get("code")+"")){
                payUrl =  jb.get("data")+"";
            }else{
                return ServerResponse.createByErrorMsg("创建支付订单失败");
            }
        }else {
            if("success".equals(msg)){
                JSONObject data = (JSONObject) jb.get("data");
                JSONObject jbb = JSON.parseObject(data.getString("result"));
                payUrl = jbb.getString("url");
            }else{
                return ServerResponse.createByErrorMsg("创建支付订单失败");
            }
        }
        GPayVO gPayVO = new GPayVO();
        gPayVO.setUrl(payUrl);
        if (insertCount > 0) {
            String email_token = KeyUtils.getUniqueKey();
            String redisSetExResult = RedisShardedPoolUtils.setEx(email_token, email_token, 300);
            log.info("用户充币，保存redis token成功，redisSetExResult = {}", redisSetExResult);
//            SendHTMLMail.send(user, userRecharge, email_token, siteInfo
//                    .getSiteHost(), siteInfo.getSiteEmailTo());
//            log.info("用户充币，发送审核邮件成功");
            messageService.sendTopicMessage(String.format("%s充币了%s", user.getRealName(), amt));
            return ServerResponse.createBySuccess(gPayVO);
        }
        return ServerResponse.createByErrorMsg("创建支付订单失败");
    }


    public ServerResponse findUserRechargeByOrderSn(String orderSn) {
        UserRecharge userRecharge = this.userRechargeMapper.findUserRechargeByOrderSn(orderSn);
        if (userRecharge != null) {
            return ServerResponse.createBySuccess(userRecharge);
        }
        return ServerResponse.createByErrorMsg("找不到充币订单");
    }


    @Transactional
    public ServerResponse chargeSuccess(UserRecharge userRecharge) throws Exception {
        log.info("充币订单 确认成功操作 id = {}", userRecharge.getId());

        if (userRecharge.getOrderStatus().intValue() != 0) {
            return ServerResponse.createByErrorMsg("订单状态不能重复修改");
        }


        User user = this.userMapper.selectByPrimaryKey(userRecharge.getUserId());
        if (user == null) {
            return ServerResponse.createByErrorMsg("用户不存在");
        }
        BigDecimal userAmt_before = user.getUserAmt();
        BigDecimal enableAmt_before = user.getEnableAmt();
        user.setUserAmt(userAmt_before.add(userRecharge.getPayAmt()));
        user.setEnableAmt(enableAmt_before.add(userRecharge.getPayAmt()));
        int updateCount = this.userMapper.updateByPrimaryKeySelective(user);
        if (updateCount > 0) {
            log.info("1.修改用户资金成功");
        } else {
            return ServerResponse.createByErrorMsg("失败，修改用户资金失败");
        }


        userRecharge.setOrderStatus(Integer.valueOf(1));
        userRecharge.setPayTime(new Date());
        int updateCCount = this.userRechargeMapper.updateByPrimaryKeySelective(userRecharge);
        if (updateCCount > 0) {
            log.info("2.修改订单状态成功");
        } else {
            throw new Exception("2. 修改订单状态失败!");
        }


        UserCashDetail ucd = new UserCashDetail();
        ucd.setAgentId(user.getAgentId());
        ucd.setAgentName(user.getAgentName());
        ucd.setUserId(user.getId());
        ucd.setUserName(user.getRealName());
        ucd.setDeType("用户充币");
        ucd.setDeAmt(userRecharge.getPayAmt());
        ucd.setDeSummary("用户充币成功，充币前总金额:" + userAmt_before + ",充币后总金额:" + user.getUserAmt() + ",充币前可用:" + enableAmt_before + ",充币后可用:" + user
                .getEnableAmt());

        ucd.setAddTime(new Date());
        ucd.setIsRead(Integer.valueOf(0));
        int insertCount = this.userCashDetailMapper.insert(ucd);
        if (insertCount > 0) {
            return ServerResponse.createBySuccessMsg("充币成功！");
        }
        return ServerResponse.createByErrorMsg("充币失败");
    }


    public ServerResponse chargeFail(UserRecharge userRecharge) throws Exception {
        if (userRecharge.getOrderStatus().intValue() != 0) {
            return ServerResponse.createByErrorMsg("订单状态不能重复修改");
        }

        userRecharge.setOrderStatus(Integer.valueOf(2));
        int updateCCount = this.userRechargeMapper.updateByPrimaryKeySelective(userRecharge);
        if (updateCCount > 0) {
            return ServerResponse.createBySuccessMsg("订单已修改为失败");
        }
        return ServerResponse.createByErrorMsg("修改出现异常");
    }


    public ServerResponse chargeCancel(UserRecharge userRecharge) throws Exception {
        if (userRecharge.getOrderStatus().intValue() != 0) {
            return ServerResponse.createByErrorMsg("订单状态不能重复修改");
        }

        userRecharge.setOrderStatus(Integer.valueOf(3));
        int updateCCount = this.userRechargeMapper.updateByPrimaryKeySelective(userRecharge);
        if (updateCCount > 0) {
            return ServerResponse.createBySuccessMsg("订单取消成功");
        }
        return ServerResponse.createByErrorMsg("订单取消出现异常");
    }


    public ServerResponse<PageInfo> findUserChargeList(String payChannel, String orderStatus, HttpServletRequest request, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        User user = this.iUserService.getCurrentUser(request);
        List<UserRecharge> userRecharges = this.userRechargeMapper.findUserChargeList(user.getId(), payChannel, orderStatus);
        PageInfo pageInfo = new PageInfo(userRecharges);
        return ServerResponse.createBySuccess(pageInfo);
    }


    public ServerResponse<PageInfo> listByAgent(Integer agentId, String realName, String payChannel, Integer state, HttpServletRequest request, int pageNum, int pageSize) {
        AgentUser currentAgent = this.iAgentUserService.getCurrentAgent(request);


        if (agentId != null) {
            AgentUser agentUser = this.agentUserMapper.selectByPrimaryKey(agentId);
            if (agentUser.getParentId() != currentAgent.getId()) {
                return ServerResponse.createByErrorMsg("不能查询非下级代理记录");
            }
        }
        Integer searchId = null;
        if (agentId == null) {
            searchId = currentAgent.getId();
        } else {
            searchId = agentId;
        }


        PageHelper.startPage(pageNum, pageSize);


        List<UserRecharge> userRecharges = this.userRechargeMapper.listByAgent(searchId, realName, payChannel, state);

        PageInfo pageInfo = new PageInfo(userRecharges);

        return ServerResponse.createBySuccess(pageInfo);
    }


    public ServerResponse listByAdmin(Integer agentId, Integer userId, String realName, Integer state, String beginTime, String endTime, HttpServletRequest request, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);


        Timestamp begin_time = null;
        if (StringUtils.isNotBlank(beginTime)) {
            begin_time = DateTimeUtil.searchStrToTimestamp(beginTime);
        }
        Timestamp end_time = null;
        if (StringUtils.isNotBlank(endTime)) {
            end_time = DateTimeUtil.searchStrToTimestamp(endTime);
        }


        List<UserRecharge> userRecharges = this.userRechargeMapper.listByAdmin(agentId, userId, realName, state, begin_time, end_time);

        PageInfo pageInfo = new PageInfo(userRecharges);

        return ServerResponse.createBySuccess(pageInfo);
    }


    public String doNotify365(String data){
        Map<String, Object> mapc = new HashMap();
        String code = "0";
        String message = "";
        String data11 = "";
        JSONObject json = JSONObject.parseObject(data);
        Map<String, Object> map = (Map<String, Object>) json;
        // 验证签名
        String sign = GpayUtil.notify365(data);
        if (!map.get("sign").toString().equals(sign)) {
            message =  "签名不一致";
        }else {
            code = "200" ;
        }
        String amount = map.get("amount").toString();
        String orderSn = map.get("orderno").toString();
        String tradeid = map.get("seller_name").toString();
        String usdt = map.get("qty").toString();
        try {
            payNotify(amount , orderSn ,tradeid ,usdt);
        }catch (Exception e){
            code =  "0";
        }
        mapc.put("code",code);
        mapc.put("message",message);
        mapc.put("data",data);
        JSONObject jsonc = new JSONObject(mapc);
        return jsonc.toString();
    }

    public ServerResponse payNotify(String amount, String orderSn ,String tradeid ,String usdt) throws Exception {
        UserRecharge userRecharge = this.userRechargeMapper.selectByOrderSn(orderSn);
        if (userRecharge == null) {
            return ServerResponse.createByErrorMsg("充值订单不存在");
        }
        if (userRecharge.getOrderStatus() != 0) {
            return ServerResponse.createByErrorMsg("订单状态不是下单状态不能更改");
        }
        User user = this.userMapper.selectByPrimaryKey(userRecharge.getUserId());
        if (user == null) {
            return ServerResponse.createByErrorMsg("用户不存在");
        }
        //总资金 = 原总资金 + 充值金额
        BigDecimal user_amt =user.getUserAmt()== null ? BigDecimal.ZERO : user.getUserAmt();
        user_amt = user_amt.add(new BigDecimal(usdt));
        log.info("支付回调确认订单成功，增加用户 {} 总资金，原金额 = {} , 增加后 = {},实际充值人民币={}", user.getId(), usdt, user_amt ,amount);
        user.setUserAmt(user_amt);
        //可用资金 = 原可用资金 + (充值金额*杠杆)
        BigDecimal user_enable_amt = user.getEnableAmt()== null ? BigDecimal.ZERO :user.getEnableAmt();
        user_enable_amt =  user_enable_amt.add(new BigDecimal(usdt));
        log.info("支付回调确认订单成功，增加用户 {} 可用资金，原金额 = {} , 增加后 = {}", user.getId(), usdt, user_enable_amt);
        user.setEnableAmt(user_enable_amt);
        int update = this.userMapper.updateByPrimaryKeySelective(user);
        if (update > 0) {
            log.info("修改用户资金成功！");
        } else {
            log.error("修改用户资金出错，抛出异常");
            throw new Exception("修改用户资金出错，抛出异常");
        }
        userRecharge.setPayAmt(new BigDecimal(usdt));
        userRecharge.setPayAmtRMB(new BigDecimal(amount));
        userRecharge.setOrderStatus(1);
        userRecharge.setPayTime(new Date());
//        userRecharge.setOrderDesc(tradeid);
        int updateCount = this.userRechargeMapper.updateByPrimaryKeySelective(userRecharge);
        if (updateCount > 0) {
            SiteTaskLog siteTaskLog = new SiteTaskLog();
            siteTaskLog.setTaskType("修改充值记录");
            String target = "GPAY回调:修改充值记录" + userRecharge.getId() +"充值金额" + amount + "用户" + userRecharge.getUserId();
            siteTaskLog.setTaskTarget(target);
            siteTaskLog.setIsSuccess(0);
            siteTaskLog.setAddTime(new Date());
            this.siteTaskLogMapper.insert(siteTaskLog);
            return ServerResponse.createBySuccessMsg("修改订单状态成功！");
        }
        return ServerResponse.createByErrorMsg("修改订单状态失败！");
    }




    @Transactional
    public ServerResponse updateState(Integer chargeId, Integer state) throws Exception {
        UserRecharge userRecharge = this.userRechargeMapper.selectByPrimaryKey(chargeId);
        if (userRecharge == null) {
            return ServerResponse.createByErrorMsg("充币订单不存在");
        }
        if (userRecharge.getOrderStatus().intValue() != 0) {
            return ServerResponse.createByErrorMsg("订单状态不是下单状态不能更改");
        }
        if (state.intValue() == 1) {
            User user = this.userMapper.selectByPrimaryKey(userRecharge.getUserId());
            if (user == null) {
                return ServerResponse.createByErrorMsg("用户不存在");
            }
            BigDecimal user_amt = user.getUserAmt().add(userRecharge.getPayAmt());
            log.info("管理员确认订单成功，增加用户 {} 总资金，原金额 = {} , 增加后 = {}", new Object[]{user.getId(), user.getUserAmt(), user_amt});
            user.setUserAmt(user_amt);
            BigDecimal user_enable_amt = user.getEnableAmt().add(userRecharge.getPayAmt());
            log.info("管理员确认订单成功，增加用户 {} 可用资金，原金额 = {} , 增加后 = {}", new Object[]{user.getId(), user.getEnableAmt(), user_enable_amt});
            user.setEnableAmt(user_enable_amt);

            int updateCount = this.userMapper.updateByPrimaryKeySelective(user);
            if (updateCount > 0) {
                log.info("修改用户资金成功！");
            } else {
                log.error("修改用户资金出错，抛出异常");
                throw new Exception("修改用户资金出错，抛出异常");
            }
        }


        userRecharge.setOrderStatus(Integer.valueOf((state.intValue() == 1) ? 1 : 2));
        userRecharge.setPayTime(new Date());
        int updateCount = this.userRechargeMapper.updateByPrimaryKeySelective(userRecharge);
        if (updateCount > 0) {
            return ServerResponse.createBySuccessMsg("修改订单状态成功！");
        }
        return ServerResponse.createByErrorMsg("修改订单状态失败！");
    }


    public ServerResponse createOrder(Integer userId, Integer state, Integer amt, String payChannel) {
        if (userId == null || state == null || amt == null) {
            return ServerResponse.createByErrorMsg("参数不能为空");
        }

        User user = this.userMapper.selectByPrimaryKey(userId);
        if (user == null) {
            return ServerResponse.createByErrorMsg("找不到用户");
        }

        UserRecharge userRecharge = new UserRecharge();
        userRecharge.setUserId(user.getId());
        userRecharge.setNickName(user.getRealName());
        userRecharge.setAgentId(user.getAgentId());
        userRecharge.setAgentName(user.getAgentName());
        userRecharge.setPhone(user.getPhone());
        String ordersn = KeyUtils.getRechargeOrderSn();
        userRecharge.setOrderSn(ordersn);

        userRecharge.setPayChannel(payChannel);
        userRecharge.setPayAmt(new BigDecimal(amt.intValue()));
        userRecharge.setAddTime(new Date());
        userRecharge.setPayTime(new Date());

        if (state.intValue() == 0) {
            userRecharge.setOrderStatus(Integer.valueOf(0));
        } else if (state.intValue() == 1) {
            userRecharge.setOrderStatus(Integer.valueOf(1));

            user.setUserAmt(user.getUserAmt().add(new BigDecimal(amt.intValue())));
            user.setEnableAmt(user.getEnableAmt().add(new BigDecimal(amt.intValue())));
            this.userMapper.updateByPrimaryKeySelective(user);
        } else if (state.intValue() == 2) {
            userRecharge.setOrderStatus(Integer.valueOf(2));
        } else {
            return ServerResponse.createByErrorMsg("订单状态不正确");
        }

        int insertCount = this.userRechargeMapper.insert(userRecharge);
        if (insertCount > 0) {
            return ServerResponse.createBySuccessMsg("生成订单成功！");
        }
        return ServerResponse.createByErrorMsg("生成订单失败，请重试");
    }


    public ServerResponse del(Integer cId) {
        if (cId == null) {
            return ServerResponse.createByErrorMsg("id不能为空");
        }
        int updateCount = this.userRechargeMapper.deleteByPrimaryKey(cId);
        if (updateCount > 0) {
            return ServerResponse.createBySuccessMsg("删除成功");
        }
        return ServerResponse.createByErrorMsg("删除失败");
    }


    public int deleteByUserId(Integer userId) {
        return this.userRechargeMapper.deleteByUserId(userId);
    }


    public BigDecimal CountChargeSumAmt(Integer chargeState) {
        return this.userRechargeMapper.CountChargeSumAmt(chargeState);
    }
}
